@@ -1,5 +1,6 @@
 //! Helper functions useful for interacting with PHP and Zend.
 
+use crate::errors::{Error, Result};
 use std::ffi::CString;
 
 /// Takes a Rust string object, converts it into a C string
@@ -14,7 +15,7 @@ use std::ffi::CString;
 /// use std::ffi::CString;
 /// use ext_php_rs::functions::c_str;
 ///
-/// let mut ptr = c_str("Hello");
+/// let mut ptr = c_str("Hello").unwrap();
 ///
 /// unsafe {
 ///     assert_eq!(b'H', *ptr as u8);
@@ -28,9 +29,16 @@ use std::ffi::CString;
 ///     let _ = CString::from_raw(ptr as *mut i8);
 /// }
 /// ```
-pub fn c_str<S>(s: S) -> *const i8
+///
+/// # Errors
+///
+/// Returns an error if the given string contains a NUL byte, which for obvious reasons cannot be
+/// contained inside a C string.
+pub fn c_str<S>(s: S) -> Result<*const i8>
 where
     S: AsRef<str>,
 {
-    CString::into_raw(CString::new(s.as_ref()).unwrap())
+    Ok(CString::into_raw(
+        CString::new(s.as_ref()).map_err(|_| Error::InvalidCString)?,
+    ))
 }
